<?php

require_once 
    dirname(dirname(__FILE__)) . DIRECTORY_SEPARATOR . 'tima.php';

if (!defined('DS')) {
    define('DS', DIRECTORY_SEPARATOR);
}
if (!defined('SIMPLE_TEST')) {
    define('SIMPLE_TEST', dirname(__FILE__) . DS . 'simpletest');
}

require_once 
    SIMPLE_TEST . DS . 'unit_tester.php';
require_once
    SIMPLE_TEST . DS . 'reporter.php';

/**
 *  ClassLoader饹Υƥȥ
 */
class Test_ClassLoader extends UnitTestCase 
{
    /**
     * åΥե롼ȡѥ
     * 
     * @var    string
     * @access private
     */
    var $_rootPath;

    /**
     * 󥹥ȥ饯
     * 
     * @access puclic
     */
    function Test_ClassLoader()
    {
        $this->UnitTestCase();

        require_once 
            dirname(dirname(__FILE__)) . DS . 'tima' . DS . 'ClassLoader.class.php';

        $this->_rootPath = dirname(__FILE__) . DS . 'ClassLoader';
   }

    /**
     * å
     * 
     * @return void
     * @access puclic
     */
    function setUp()
    {
        @mkdir($this->_rootPath);
        @mkdir($this->_rootPath . DS . 'Child');
        @mkdir($this->_rootPath . DS . 'Child' . DS . 'Grandchild');
        @mkdir($this->_rootPath . DS . 'dummy_dir');

        chdir($this->_rootPath);

        $mock_class = '<?php class MockClass1 {}';
        $handle = fopen($this->_rootPath . DS . 'MockClass1.class.php', 'w');
        fwrite($handle, $mock_class);
        fclose($handle);

        $mock_class = '<?php class MockClass2 {}';
        $handle = fopen($this->_rootPath . DS . 'MockClass2.php', 'w');
        fwrite($handle, $mock_class);
        fclose($handle);

        $mock_class = '<?php class ErrorMockClass {}';
        $handle = fopen($this->_rootPath . DS . 'MockClass.class.php', 'w');
        fwrite($handle, $mock_class);
        fclose($handle);

        $mock_class = '<?php class MockClass3 {}';
        $handle = 
            fopen(
                $this->_rootPath . DS . 'Child' . DS . 'MockClass3.class.php', 
                'w');
        fwrite($handle, $mock_class);
        fclose($handle);

        $mock_class = '<?php class Child_MockClass4 {}';
        $handle = 
            fopen(
                $this->_rootPath . DS . 'Child' . DS . 'MockClass4.phtml', 
                'w');
        fwrite($handle, $mock_class);
        fclose($handle);

        $mock_class = '<?php class Child_Grandchild_MockClass5 {}';
        $handle = 
            fopen(
                $this->_rootPath . DS . 'Child' . DS . 
                    'Grandchild' . DS . 'MockClass5.class.php', 
                'w');
        fwrite($handle, $mock_class);
        fclose($handle);
    }

    /**
     * åΤΥեǥ쥯ȥݽ
     * 
     * @return void
     * @access puclic
     */
    function tearDown()
    {
        $this->_cleanDir($this->_rootPath);
    }

    /**
     * 󥹥ȥ饯Υƥ
     * 
     * @return void
     * @access puclic
     */
    function testConstructor()
    {
        // 1˥ľΡ*.class.phpץեΥ饹
        $loader_a = &new ClassLoader;
        $this->assertTrue($loader_a->isReadable('MockClass1'));

        // 2˥ľΡ*.phpץեΥ饹
        $loader_b = &new ClassLoader('', '.php');
        $this->assertTrue($loader_b->isReadable('MockClass2'));

        // 3ˡChildץǥ쥯ȥ겼Ρ*.class.phpץեΥ饹
        $loader_c = &new ClassLoader('Child');
        $this->assertTrue($loader_c->isReadable('MockClass3'));

        // 4ˡChildץǥ쥯ȥ겼Ρ*.phtmlץեΥ饹
        $loader_d = &new ClassLoader('Child', '.phtml');
        $this->assertTrue($loader_d->isReadable('MockClass4'));

        $dummy_path = $this->_rootPath . DS . 'dummy_dir';

        // 1ˤƱɤ߹Ū
        $loader_e = &new ClassLoader('', '.class.php', $this->_rootPath);
        $this->assertTrue($loader_e->isReadable('MockClass1'));

        // 1ˤƱɤ߹
        $loader_e = &new ClassLoader('', '.class.php', $dummy_path);
        $this->assertFalse($loader_e->isReadable('MockClass1'));
    }

    /**
     * ClassLoader::load() ᥽åɤΥƥ
     * 
     * @return void
     * @access puclic
     */
    function testLoad()
    {
        $loader = &new ClassLoader;

        // ƻҴطΤʤ饹Υ
        $this->assertFalse(class_exists('MockClass1'));
        $this->assertEqual('MockClass1', $loader->load('MockClass1'));
        $this->assertTrue(class_exists('MockClass1'));

        // ¸ߤʤ饹Υ
        $this->assertEqual('', $loader->load('NotExistsClass'));

        // ե̾ȥ饹̾פʤ饹Υ
        // - ¸ŪʥХʤ̵ۤ
        //  - ErrorMockClass饹ϡMockClass.class.phpפƤ
        //   1. ErrorMockClassloadϥե뤬Ժ
        //   2. MockClassloadϥ饹̾
        //   3. MockClassloadԻErrorMockClassɤ߹ޤƤ
        //      ErrorMockClassload
        $this->assertEqual('', $loader->load('ErrorMockClass'));
        $this->assertEqual('', $loader->load('MockClass'));
        $this->assertEqual('ErrorMockClass', $loader->load('ErrorMockClass'));
    }

    /**
     * ClassLoader::isReadable() ᥽åɤΥƥ
     * 
     * @return void
     * @access puclic
     */
    function testIsReadable()
    {
        $loader_a = &new ClassLoader;
        $this->assertTrue($loader_a->isReadable('MockClass1'));
        $this->assertFalse($loader_a->isReadable('MockClass2'));
        $this->assertFalse($loader_a->isReadable('MockClass3'));
        $this->assertFalse($loader_a->isReadable('MockClass4'));
        $this->assertTrue($loader_a->isReadable('MockClass'));

        $loader_b = &new ClassLoader('', '.php');
        $this->assertFalse($loader_b->isReadable('MockClass1'));
        $this->assertTrue($loader_b->isReadable('MockClass2'));
        $this->assertFalse($loader_b->isReadable('MockClass3'));
        $this->assertFalse($loader_b->isReadable('MockClass4'));
        $this->assertFalse($loader_b->isReadable('MockClass'));

        $loader_c = &new ClassLoader('Child');
        $this->assertFalse($loader_c->isReadable('MockClass1'));
        $this->assertFalse($loader_c->isReadable('MockClass2'));
        $this->assertTrue($loader_c->isReadable('MockClass3'));
        $this->assertFalse($loader_c->isReadable('MockClass4'));
        $this->assertFalse($loader_c->isReadable('MockClass'));

        $loader_d = &new ClassLoader('Child', '.phtml');
        $this->assertFalse($loader_d->isReadable('MockClass1'));
        $this->assertFalse($loader_d->isReadable('MockClass2'));
        $this->assertFalse($loader_d->isReadable('MockClass3'));
        $this->assertTrue($loader_d->isReadable('MockClass4'));
        $this->assertFalse($loader_d->isReadable('MockClass'));
    }

    /**
     * ClassLoader::setParents() ᥽åɤΥƥ
     * 
     * @return void
     * @access puclic
     */
    function testSetParents()
    {
        $loader = &new ClassLoader;

        $this->assertTrue($loader->isReadable('MockClass1'));
        $this->assertFalse($loader->isReadable('MockClass3'));
        $this->assertFalse($loader->isReadable('MockClass5'));

        $loader->setParents('Child');
        $this->assertFalse($loader->isReadable('MockClass1'));
        $this->assertTrue($loader->isReadable('MockClass3'));
        $this->assertFalse($loader->isReadable('MockClass5'));

        $loader->setParents('Child', 'Grandchild');
        $this->assertFalse($loader->isReadable('MockClass1'));
        $this->assertFalse($loader->isReadable('MockClass3'));
        $this->assertTrue($loader->isReadable('MockClass5'));
    }

    /**
     * ClassLoader::setFileExt() ᥽åɤΥƥ
     * 
     * @return void
     * @access puclic
     */
    function testSetFileExt()
    {
        $loader = &new ClassLoader;

        $loader->setFileExt('.txt');
        $this->assertFalse($loader->isReadable('MockClass1'));
        $this->assertFalse($loader->isReadable('MockClass2'));

        $loader->setFileExt('.class.php');
        $this->assertTrue($loader->isReadable('MockClass1'));
        $this->assertFalse($loader->isReadable('MockClass2'));

        $loader->setFileExt('.php');
        $this->assertFalse($loader->isReadable('MockClass1'));
        $this->assertTrue($loader->isReadable('MockClass2'));

        $loader->setFileExt('class.php');
        $this->assertFalse($loader->isReadable('MockClass1'));
        $this->assertFalse($loader->isReadable('MockClass2'));

        $loader->setFileExt('php');
        $this->assertFalse($loader->isReadable('MockClass1'));
        $this->assertFalse($loader->isReadable('MockClass2'));
    }

    /**
     * Υǥ쥯ȥȤ۲ΥեƵŪ˺
     * 
     * @return void
     * @access private
     */
    function _cleanDir($path)
    {
        if (($dh = @opendir($path)) === false) {
            return;
        }

        while (($file = @readdir($dh)) !== false) {
            if ($file == '.' || $file == '..') {
                continue;
            }
            if (is_dir($path . DS . $file)) {
                $this->_cleanDir($path . DS . $file);
                continue;
            }
            @unlink($path . DS .  $file);
        }
        closedir($dh);

        @rmdir($path);
    }
}

$test = &new Test_ClassLoader;
$test->run(new TextReporter);
