<?php

/**
 * The tiny modules for web application
 * - PHP versions 4 -
 * 
 * @category  web application framework
 * @package   tima
 * @author    IKEDA Youhey <youhey.ikeda@gmail.com>
 * @license   http://www.apache.org/licenses/LICENSE-2.0 Apache License, Version 2.0
 * @copyright 2007 IKEDA Youhey
 *     Licensed under the Apache License, Version 2.0 (the "License"); 
 *     you may not use this file except in compliance with the License. 
 *     You may obtain a copy of the License at 
 *         http://www.apache.org/licenses/LICENSE-2.0 
 *     Unless required by applicable law or agreed to in writing, software 
 *     distributed under the License is distributed on an "AS IS" BASIS, 
 *     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *     See the License for the specific language governing permissions and 
 *     limitations under the License.
 * @version  1.0.0
 */

/**
 * å򳫻ϡλ뤿Υե륿
 * 
 * @package    tima
 * @subpackage tima_Filter
 * @author     IKEDA Youhey <youhey.ikeda@gmail.com>
 */
class Filter_SessionGateway
{

    /**
     * åγ
     * 
     * @param  Front $front
     * @return void
     * @access public
     */
    function before(&$front)
    {
        $logger     = &$front->getLogger();
        $config     = &$front->getConfig();
        $user_agent = &$front->getUserAgent();
        $use_cookie = false;
        if (!$user_agent->isMobile()) {
            $use_cookie = $config->get('use_cookie', 'session');
        }
        ini_set('session.use_cookies', $use_cookie);
        ini_set('session.use_only_cookies', $use_cookie);

        $name     = $config->get('session_name', 'env');
        $lifetime = $config->get('cookie_lifetime', 'session');
        $path     = $config->get('cookie_path', 'session');
        $domain   = $config->get('cookie_domain', 'session');
        $secure   = $config->get('cookie_secure', 'session');

        $session = $front->getSession();
        $session->start($name, $lifetime, $path, $domain, $secure);
        $logger->debug('Session was started');
        $logger->debug('Session-id was '.$session->getId());
        $logger->debug('Session was cookie are '.($use_cookie?'used':'not used'));
        $logger->debug("Session used the name of ${name}");
        $logger->debug("Session used the lifetime of ${lifetime}");
        $logger->debug("Session used the root-path of ${path}");
        $logger->debug("Session used the domain of ${domain}");
        $logger->debug('Session used the secure of '.($secure?'Yes':'No'));

        $logger->trace('Executed pre filtering: '.__CLASS__);
    }

    /**
     * åνλ
     *
     * @param  Front $front
     * @return void
     * @access public
     */
    function after(&$front)
    {
        $logger = &$front->getLogger();

        $session = $front->getSession();
        if ($session->isStarted()) {
            $session->stop();
            $logger->debug('Session was stopped');
        }

        $logger->trace('Executed post filtering: '.__CLASS__);
    }
}
