<?php

/**
 * The tiny modules for web application
 * - PHP versions 4 -
 * 
 * @category  web application framework
 * @package   tima
 * @author    IKEDA Youhey <youhey.ikeda@gmail.com>
 * @license   http://www.apache.org/licenses/LICENSE-2.0 Apache License, Version 2.0
 * @copyright 2007 IKEDA Youhey
 *     Licensed under the Apache License, Version 2.0 (the "License"); 
 *     you may not use this file except in compliance with the License. 
 *     You may obtain a copy of the License at 
 *         http://www.apache.org/licenses/LICENSE-2.0 
 *     Unless required by applicable law or agreed to in writing, software 
 *     distributed under the License is distributed on an "AS IS" BASIS, 
 *     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *     See the License for the specific language governing permissions and 
 *     limitations under the License.
 * @version  1.0.0
 */

/**
 * եȡȥ
 * 
 * @package    tima
 * @subpackage tima_Front
 * @author     IKEDA Youhey <youhey.ikeda@gmail.com>
 */
class Front
{

    /**
     * ƥץ졼ȡեγĥ
     * 
     * @var    string
     * @access protected
     */
    var $templateExt = '.html';

    /**
     * ¹ԥץ
     * 
     * @var    Process
     * @access private
     */
    var $_process = null;

    /**
     * ӥ塼
     * 
     * @var    View
     * @access private
     */
    var $_view = null;

    /**
     * ꥯ
     * 
     * @var    Request
     * @access private
     */
    var $_request = null;

    /**
     * å
     * 
     * @var    session
     * @access private
     */
    var $_session = null;

    /**
     * 쥹ݥ
     * 
     * @var    Response
     * @access private
     */
    var $_response = null;

    /**
     * 桼
     * 
     * @var    UserAgent
     * @access private
     */
    var $_userAgent = null;

    /**
     * ե륿
     * 
     * @var    array
     * @access private
     */
    var $_filters = array();

    /**
     * 
     * 
     * @var    Config
     * @access private
     */
    var $_config = null;

    /**
     * 
     * 
     * @var    Logger
     * @access private
     */
    var $_log = null;

    /**
     * ưϻ
     * 
     * @var    float
     * @access private
     */
    var $_beginning = 0.0;

    /**
     * 󥹥ȥ饯
     * - 㳰ȯ۾ｪλ
     *  - 饹¸ߤʤ
     *   - 顼Logger '̾' not found
     *   - 쥹ݥ󥹡500 Internal Server Error
     * 
     */
    function Front()
    {
        list($u,$s) = explode(" ", microtime());
        $this->_beginning = ((float)$u + (float)$s);
        $this->_process   = &new Process;
        $this->_request   = &new Request;
        $this->_response  = &new Response;
        $this->_config    = &new Config;
        $this->_userAgent = &new UserAgent;

        // եɤ߹
        $root_etc = ROOT_PATH . DS . 'etc';
        $app_etc  = $this->getAppDir() . DS . 'etc';
        $this->_config->setEtcDir($root_etc);
        if ($root_etc !== $app_etc) {
            $this->_config->setEtcDir($app_etc);
        }
        $reading_result = $this->_config->readConfig('all');
        if ($reading_result !== true) {
            trigger_error(
                "Unable to read the configuration 'all-ini.php'", 
                E_USER_WARNING);
        }

        // ȥͭեɤ߹
        $this->_config->readConfig(get_class($this));

        // å
        $this->_session = &new Session('__Front');

        // ӥ塼
        $basedir = $this->getAppDir() . DS . 'templates';
        $device  = $this->_userAgent->isMobile() ? 'mobile' : 'pc';
        $this->_view = 
            &new Smarty4View(
                $this->getInternalEncoding(), $this->getContentsEncoding(), 
                Utility::merge(
                    $this->_config->getByNamespace('template'), 
                    array(
                        'template_dir' => $basedir . DS . 'template' . DS . $device, 
                        'compile_dir'  => $basedir . DS . 'templates_c')));
        $this->_view->registerObject('Request', $this->_request);
        $this->_view->registerObject('UserAgent', $this->_userAgent);

        // 
        $logger_class = $this->_config->get('logger_class', 'env');
        if (($logger_class === null) || !class_exists($logger_class)) {
            header('HTTP/1.0 500 Internal Server Error');
            trigger_error("Logger '${logger_class}' not found", E_USER_ERROR);
            exit;
        }
        $this->_log = 
            &new $logger_class(
                $this->_config->get('log_level', 'env'), 
                $this->_config->get('log_option', 'env'));
    }

    /**
     * ȥʥѡ饹Ǥ϶
     * 
     * Ѿ饹ǽФ򥪡С饤
     * νϥ¹Ԥΰֺǽ˸ƤӽФ
     * 
     * @return void
     * @access public
     */
    function initialize() {}

    /**
     * եȡȥ¹
     * - ¹Էײ褫鿷ƥץ¹
     * - 㳰ȯ۾ｪλ
     *  - 롼ƥ󥰤˼ԡü¹Էײ褬
     *   - 顼Unable to dispatch process in mapping
     *   - 쥹ݥ󥹡500 Internal Server Error
     * 
     * @param  array $mapping
     * @return void
     * @access public
     * @see    Front::route()
     * @see    Front::dispatch()
     * @see    Front::process()
     * @see    Front::_applyFiltersBeforeMethod()
     * @see    Front::_applyFiltersAfterMethod()
     */
    function start($mapping = null)
    {
        $this->_log->info('##### The tima started operations (FRONT CTRL) #####');

        $this->initialize();

        $this->_applyFiltersBeforeMethod();

        if ($mapping === null) {
            $mapping = $this->route();
        }

        if (!is_array($mapping) || !isset($mapping['path'])) {
            $this->_response->clearHeader();
            $this->_response->setStatus('500');
            $this->_log->warn('Unable to dispatch process in mapping');
        } else {
            if (!array_key_exists('method', $mapping)) {
                $mapping['method'] = null;
            }
            $this->_log->debug("Starting mapping path: {$mapping['path']}");
            $this->_log->debug("Starting mapping method: {$mapping['method']}");


            $this->dispatch($mapping['path'], $mapping['method']);
            $this->_config->readConfig($mapping['path']);
            $this->process();
        }

        $this->_applyFiltersAfterMethod();

        $this->_response->flush();
    }

    /**
     * 쥯
     * 
     * 쥹ݥ󥹤˥쥯ȤΤΥإåϿ
     * 쥹ݥ󥹤˥쥯ȤΤΥƥĤϿ
     * - Υ᥽åɤľܥ쥯Ƚȯʤ
     * - ¸Υ쥹ݥ󥹾쥯Ȥξѹ
     *  - ᥽åɤθƤӽФǥ쥹ݥƤ񤭴Х쥯Ȥ̵
     *  - ¸Υ쥹ݥ󥹾ϥ쥯Ⱦǽ񤭴Ƽ
     * - ǥեȤΥơɤϡ302 Moved Temporarily
     *  - RFC2616λͤǤ302ơɤǤϥꥯȤȯԾѤʤ
     *  - POSTΥꥯȤGETǥ쥯Ȥ뤳ȤϤǤʤ
     *  - Ūʥ桼ȤǤ302ơɤǤ̤ư
     *  - POSTΥꥯȤGETǥ쥯ȤƤٹʤȯʤ
     * - CGIʤɤǤΥ쥯Ȥˤϡ303 See OtherפѤ٤
     *  - 桼ȤˤäƤбƤʤΤ
     *  - бƤ桼ȤǤХ򸫤
     * - Ūʥ桼ȤȤθߴθ302ơɤ
     * - Ū˷ٹʤɽФ桼ȤФƤƹ
     * 
     * @param  string $link
     * @param  array  $params
     * @param  string $status_code
     * @return void
     * @access public
     */
    function redirect($link, $params = array(), $status_code = '302')
    {
        $url = htmlentities($link, ENT_QUOTES, $this->getHttpCharSet());

        if (is_array($params) && !empty($params)) {
            $query = array();
            foreach ($params as $varkey => $varvalue) {
                $query[] = urlencode($varkey).'='.urlencode($varvalue);
            }
            $url .= '?'.implode('&', $query);
        }

        $this->_response->clearHeader();
        $this->_response->setStatus($status_code);
        $this->_response->setHeader('Location', $url);

        $this->_log->debug("Set the redirect URL: ${url}");

        $this->_response->setContents(
            "<html><head>\n" . 
            "<meta http-equiv=\"refresh\" content=\"0;url=${url}\" />\n" . 
            "</head></html>\n");
    }

    /**
     * ư楳ȥǤդΥ˥쥯
     * 
     * ѥ᡼ǤեλϿ
     * å󤬳ϤƤС˱ƥå󡦥ѥ᡼ɲ
     *  - äǤ̵ǥå󥭡ѥ᡼ɲ
     *  - use_cookieǤʤХå󥭡ѥ᡼ɲ
     * 
     * @param  string     $action
     * @param  array|null $params
     * @return void
     * @access public
     * @see    Front::redirect()
     */
    function redirectAction($action, $params = array())
    {
        // 
        $query = array(
                $this->_config->get('action_key', 'env') => $action, 
            );

        $this->_log->debug("Set the redirect action: ${action}");

        // å
        if ($this->_session->isStarted()) {
            $use_cookie = $this->_config->get('use_cookie', 'session');
            if ($this->_userAgent->isMobile() || ($use_cookie !== true)) {
                $query[$this->_session->getSessionName()] = $this->_session->getId();
                $this->_log->debug('Add session id for URL');
            }
        }

        $this->redirect($this->_request->getUrl(), Utility::merge($query, $params));
    }

    /**
     * ե륿Ͽ
     * - 㳰ȯ۾ｪλ
     *  - ե륿ɤ߹ʤ
     *   - 顼Filter 'ե륿̾' not found
     *   - 쥹ݥ󥹡500 Internal Server Error
     * 
     * @param  string $filter
     * @return void
     * @access public
     * @static ClassLoader $class_loader
     * @see    ClassLoader::load()
     */
    function setFilter($filter)
    {
        static $class_loader;
        if (!isset($class_loader)) {
            $class_loader = new ClassLoader;
            $class_loader->setParents('Filter');
            $class_loader->setIncludePath(ROOT_PATH);
        }

        $class_name = $class_loader->load($filter);
        if ($class_name === '') {
            $this->_log->warn("Filter '${filter}' not found");
        } else {
            $this->_filters[] = &new $class_name;
        }
    }

    /**
     * ץꥱ֤ǥ쥯ȥХѥֵ
     * 
     * @return string
     * @access public
     */
    function getAppDir()
    {
        $app_dir = '';

        $varvalue = defined('APP_DIR') ? APP_DIR : ROOT_PATH;
        if (OS_WINDOWS) {
            if (preg_match('/^[a-z]:/i', $varvalue) && ($varvalue{2} === DS)) {
                $app_dir = $varvalue;
            }
        } elseif ($varvalue{0} === DS) {
            $app_dir = $varvalue;
        }
        if ($app_dir === '') {
            $app_dir = realpath(ROOT_PATH . DS . $varvalue);
        }

        return $app_dir;
    }

    /**
     * ʸ󥳡ǥ󥰤ֵ
     * 
     * ʸ󥳡ǥ󥰤̵ͭˤäưѲ
     * - ꤢꡧꤵ줿ֵͤ
     * - ʤPHPưĶϿ => Ͽֵͤ
     *  - ưĶο¬ˤmb_internal_encoding()ؿ
     *  - ѹˤäƤʤʤ⤢
     * 
     * @return string|null
     * @access public
     */
    function getInternalEncoding()
    {
        $internal_encoding = $this->_config->get('internal_encoding', 'env');
        if ($internal_encoding === null) {
            $internal_encoding = mb_internal_encoding();
        }

        return $internal_encoding;
    }

    /**
     * Ϥʸ󥳡ǥ󥰤ֵ
     * 
     * ̵ͭˤäưѲ
     * - ꤢꡧꤵ줿ֵͤ
     * - ʤPHPưĶϿ => Ͽֵͤ
     *  - ưĶο¬ˤmb_http_output()ؿ
     *  - ꤬ưʤʤ⤢
     * 
     * @return string|null
     * @access public
     */
    function getContentsEncoding()
    {
        $http_encoding = $this->_config->get('contents_encoding', 'env');
        if ($http_encoding === null) {
            $http_encoding = mb_http_output();
        }

        return $http_encoding;
    }

    /**
     * ϤHTMLΥ饯åȤֵ
     * 
     * @return string|null
     * @access public
     * @see    Front::getContentsEncoding()
     */
    function getHttpCharSet()
    {
        return 
            mb_preferred_mime_name($this->getContentsEncoding());
    }

    /**
     * ꥯȤֵ
     * 
     * @return Request
     * @access public
     */
    function &getRequest()
    {
        return $this->_request;
    }

    /**
     * åֵ
     * 
     * @return Session
     * @access public
     */
    function &getSession()
    {
        return $this->_session;
    }

    /**
     * 쥹ݥ󥹤ֵ
     * 
     * @return Response
     * @access public
     */
    function &getResponse()
    {
        return $this->_response;
    }

    /**
     * 桼Ȥֵ
     * 
     * @return Session
     * @access public
     */
    function &getUserAgent()
    {
        return $this->_userAgent;
    }

    /**
     * ץֵ
     * 
     * @return Process
     * @access public
     */
    function &getProcess()
    {
        return $this->_process;
    }

    /**
     * ƥץ졼ȡ󥸥ֵ
     * 
     * @return object|null
     * @access public
     */
    function &getTemplateEngine()
    {
        return $this->_view->getEngine();
    }

    /**
     * ֵ
     * 
     * @return Config
     * @access public
     */
    function &getConfig()
    {
        return $this->_config;
    }

    /**
     * ֵ
     * 
     * @return Logger
     * @access public
     */
    function &getLogger()
    {
        return $this->_log;
    }

    /**
     * ưϻֵ
     * 
     * @return float
     * @access public
     */
    function getBeginning()
    {
        return $this->_beginning;
    }

    /**
     * ꥯȤURIȥѥ᡼¹Էײ
     * - ꥯURI饢󡦥ȥΥѥ
     *  - /foo/bar/hoge.php => test_html_foo_bar_hoge
     * - ꥯͤ¹ԥ̾
     *  - GET::action::bar => bar
     * - web_root_pathꤵ줿ѥϩƬ
     *  - '/'/test/html/foo/bar/hoge.php => test_html_foo_bar_hoge
     *  - '/test/html/'/test/html/foo/bar/hoge.php => foo_bar_hoge
     * - ɥåȡ.˰ʹߤեĥҤȽǤƺ
     *  - /foo/bar/hoge => foo_bar_hoge
     *  - /foo/bar/hoge.php => foo_bar_hoge
     *  - /foo/bar/hoge.html => foo_bar_hoge
     * - ե̾ά줿ϡindexפȤƽ
     * 
     * @return array
     * @access protected
     */
    function route()
    {
        $rootpath = 
            $this->_config->get(
                $this->_request->isSecure() ? 
                    'https_root_path' : 'http_root_path', 
                'env');
        if (($rootpath === null) || !is_string($rootpath) || ($rootpath === '')) {
            $rootpath = '/';
        }
        if ($rootpath{0} !== '/') {
            $rootpath = '/' . $rootpath;
        }

        $pathuri  = $this->_request->getPathUri();

        if (strpos($pathuri, $rootpath) === 0) {
            $start   = strlen($rootpath);
            $pathuri = substr($pathuri, $start, (strlen($pathuri) - $start));
        }
        if (strpos($pathuri, '.') !== false) {
            $pathuri = substr($pathuri, 0, strpos($pathuri, '.'));
        }

        if (($pathuri === '') || ($pathuri === false)) {
            $pathuri = 'index';
        } else {
            $i = strlen($pathuri) - 1;
            if ($pathuri{$i} === '/') {
                $pathuri .= 'index';
            }
        }

        return 
            array(
                    'path' => 
                        preg_replace(
                            '/[^a-z0-9_]/i', '', 
                            str_replace(
                                array('/', '-'), '_', 
                                trim(strtolower($pathuri), '/'))), 
                    'method' => 
                        $this->_request->getAcceptRequest(
                            $this->_config->get('action_key', 'env'), 
                            array('p', 'g')));
    }

    /**
     * ¹Էײ褫饿ƥץϿ
     * 
     * 㳰ȯ۾ｪλ
     * - ꥯȤб󡦥ȥ餬¸ߤʤ
     *  - 顼Controller 'ȥ̾' not found
     *  - 쥹ݥ󥹡404 Not Found
     * - 󡦥ȥΥɤ˼
     *  - 顼"Unable to use the Controller 'ȥ̾'
     *  - 쥹ݥ󥹡404 Not Found
     * 
     * @param  string $ctrlpath 󡦥ȥΥѥ
     * @param  string $action   ¹Ԥ륢
     * @return void
     * @access protected
     * @see    Front::_findCtrl()
     * @see    ClassLoader::load()
     */
    function dispatch($ctrlpath, $action)
    {
        $class_loader = &new ClassLoader;
        $class_loader->setParents('Action');
        $class_loader->setIncludePath($this->getAppDir());

        $discovery = $this->_findCtrl($ctrlpath, $class_loader);
        if ($discovery === null) {
            header('HTTP/1.0 404 Not Found');
            trigger_error("Action-controller '${ctrlpath}' not found", E_USER_ERROR);
            exit;
        }

        $action_class = $class_loader->load($discovery['ctrl']);
        if ($action_class === '') {
            header('HTTP/1.0 404 Not Found');
            trigger_error(
                "Unable to load the action-controller '${ctrlpath}'", 
                E_USER_ERROR);
            exit;
        }

        $task = &new $action_class($this);
        $this->_process->set($task, $action, $discovery);
    }

    /**
     * ץΥ¹ԤƼ¹Է̤б
     * 
     * @return void
     * @access protected
     * @see    Process::invoke()
     * @see    Front::redirectAction()
     * @see    Front::redirect()
     */
    function process()
    {
        // ץ¹
        $reply = $this->_process->invoke();

        // ¹Է̤ɾ
        switch (true) {
        case preg_match('/^action::(.+)$/i', $reply, $match) : 
            $this->redirectAction($match[1]);
            break;
        case preg_match('/^redirect::(.+)$/i', $reply, $match) : 
            $this->redirect($match[1]);
            break;
        case ($reply !== '') : 
            $template_file = $reply . $this->templateExt;

            if (!$this->_view->isTemplateExists($template_file)) {
                header('HTTP/1.0 500 Internal Server Error');
                trigger_error("Template '${template_file}' not found", E_USER_ERROR);
                exit;
            }

            $this->_response->setContents(
                $this->_view->render(
                    $template_file, 
                    $this->_response->getDataModel()));
            break;
        }
    }

    /**
     * ¹ե륿
     * 
     * @return void
     * @access private
     */
    function _applyFiltersBeforeMethod()
    {
        for ($i = 0, $n = count($this->_filters); $i < $n; ++$i) {
            if (method_exists($this->_filters[$i], 'before')) {
                $this->_filters[$i]->before($this);
            }
        }
    }

    /**
     * ¹Ըե륿
     * 
     * @return void
     * @access private
     */
    function _applyFiltersAfterMethod()
    {
        for ($i = (count($this->_filters) - 1); $i >= 0; --$i) {
            if (method_exists($this->_filters[$i], 'after')) {
                $this->_filters[$i]->after($this);
            }
        }
    }

    /**
     * 󡦥ȥΥѥ饯饹̾򸡺
     * 
     * ѥfoo_bar_hoge_moge_abcפõե
     * 1. __action__/FooBarHogeMogeAbc.class.php
     * 2. __action__/foo/BarHogeMogeAbc.class.php
     * 3. __action__/foo_bar/HogeMogeAbc.class.php
     * 4. __action__/foo_bar_hoge/MogeAbc.class.php
     * 5. __action__/foo_bar_hoge_moge/Abc.class.php
     * 6. ԡnullֵѡ
     * 
     * @param  string      $ctrlpath
     * @param  ClassLoader $class_loader
     * @return array|null
     * @access private
     * @see    Utility::camelize()
     * @see    ClassLoader::isReadable()
     */
    function _findCtrl($ctrlpath, &$class_loader)
    {
        $ctrl_class = null;
        $classname  = Utility::camelize($ctrlpath);

        if ($class_loader->isReadable($classname)) {
            $ctrl_class = array('dir' => '', 'ctrl' => $classname);
        } else {
            $subname  = $ctrlpath;
            $routelen = strlen($ctrlpath);
            while ($subname = strstr($subname, '_')) {
                $sublen  = strlen($subname);
                $subname = substr($subname, 1, $sublen - 1);
                $dirname = substr($ctrlpath, 0, $routelen - $sublen);

                $class_loader->setParents('Action', $dirname);
                $classname = Utility::camelize($subname);

                if ($class_loader->isReadable($classname)) {
                    $ctrl_class = array('dir' => $dirname, 'ctrl' => $classname);
                    break;
                }
            }
        }

        return $ctrl_class;
    }
}
