<?php

/**
 * The tiny modules for web application
 * - PHP versions 4 -
 * 
 * @category  web application framework
 * @package   tima
 * @author    IKEDA Youhey <youhey.ikeda@gmail.com>
 * @license   http://www.apache.org/licenses/LICENSE-2.0 Apache License, Version 2.0
 * @copyright 2007 IKEDA Youhey
 *     Licensed under the Apache License, Version 2.0 (the "License"); 
 *     you may not use this file except in compliance with the License. 
 *     You may obtain a copy of the License at 
 *         http://www.apache.org/licenses/LICENSE-2.0 
 *     Unless required by applicable law or agreed to in writing, software 
 *     distributed under the License is distributed on an "AS IS" BASIS, 
 *     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *     See the License for the specific language governing permissions and 
 *     limitations under the License.
 * @version  1.0.0
 */

/**
 * 饤ȤΥꥯȾ
 * 
 * @package    tima
 * @subpackage tima_Request
 * @author     IKEDA Youhey <youhey.ikeda@gmail.com>
 * @todo       URI|URL|pathۣʤΤΤ
 *             եΥåץɤб
 */
class Request
{

    /**
     * ꥯ
     * 
     * @var    array
     * @access private
     */
    var $_parameters = array();

    /**
     * 󥹥ȥ饯
     * 
     * ꥯѿϰǻꤵ줿᥽åɤͭˤʤ
     * ά줿ˤϥꥯȡ᥽åɤΤߤоݤˤ
     * 1. g$_GET
     * 2. p$_POST
     * 
     * @param  array $method
     * @access public
     */
    function Request($method = null)
    {
        if ($method === null) {
            $method = array();
                if ($this->isPost()) $method = array('p');
            elseif ($this->isGet())  $method = array('g');
        }
        if (!is_array($method)) {
            $method = (array)$method;
        }

        foreach ($method as $section) {
            switch ($section) {
            case 'g' : $this->_fetchGetParameters();
                break;
            case 'p' : $this->_fetchPostParameters();
                break;
            }
        }
    }

    /**
     * ꥯѿֵͤ
     * 
     * @param  string $varkey
     * @return mixed
     * @access public
     */
    function get($varkey)
    {
        $varvalue = null;
        if (isset($this->_parameters[$varkey])) {
            $varvalue = $this->_parameters[$varkey];
        }

        return $varvalue;
    }

    /**
     * ꥯѿͤ
     * 
     * @param  string $varkey
     * @param  mixed  $varvalue
     * @return void
     * @access public
     */
    function set($varkey, $varvalue)
    {
        if ($this->exists($varkey)) {
            $this->_parameters[$varkey] = $varvalue;
        } else {
            trigger_error("Nonexistent request::${varkey}", E_USER_NOTICE);
        }
    }

    /**
     * ꤵ줿ѿ̾ΥꥯȤ¸ߤ뤫򸡺
     * 
     * ͤnullʤ¸ߤʤפɾ
     * ͤnullϿ¸ߤõȽ
     * 
     * @param  string $varkey
     * @return boolean
     * @access public
     */
    function exists($varkey)
    {
        return 
            isset($this->_parameters[$varkey]);
    }

    /**
     * ƤΥꥯȤֵ
     * 
     * @return array
     * @access public
     */
    function getAll()
    {
        return $this->_parameters;
    }

    /**
     * ꥯȤѿֵ̾
     * 
     * @return array
     * @access public
     */
    function getNames()
    {
        return 
            array_keys($this->_parameters);
    }

    /**
     * ᥽åɤΥꥯȤֵ
     * 
     * PHPΥꥯѿľܥ
     * ե졼Υե륿󥰤ʤɤ˱ƶʤǡ
     * 
     * ɤ߹᥽åɤǽ
     * ᥽å̤Ʊ̾ѿ¸ߤǾ
     * ǥեȤǤϡGETPOSTפνɾ
     * 1. g$_GET
     * 2. p$_POST
     * 3. c$_COOKIE
     * 4. q$_SERVER['QUERY_STRING']
     * 
     * @param  string $varkey
     * @param  array  $order
     * @return mixed
     * @access public
     */
    function getAcceptRequest($varkey, $order = array('g', 'p'))
    {
        if (!is_array($order)) {
            $order = (array)$order;
        }

        $varvalue = null;
        foreach ($order as $method) {
            if ($method === 'g') {
                if (isset($_GET[$varkey])) {
                    $varvalue = $_GET[$varkey];
                }
            } elseif ($method === 'p') {
                if (isset($_POST[$varkey])) {
                    $varvalue = $_POST[$varkey];
                }
            } elseif ($method === 'c') {
                if (isset($_COOKIE[$varkey])) {
                    $varvalue = $_COOKIE[$varkey];
                }
            } elseif ($method ===  'q') {
                $query_string = $this->getEnv('QUERY_STRING');
                if ($query_string !== null) {
                    parse_str($query_string, $params);
                    if (isset($params[$varkey])) {
                        $varvalue = $params[$varkey];
                    }
                }
            }
        }

        return $varvalue;
    }

    /**
     * ĶѿPATH_INFOפϢֵ
     * - /example.php/foo/1/bar/2/hoge/3 => array('foo'=>1, 'bar'=>2, 'hoge'=>3)
     * 
     * @return array
     * @access public
     */
    function getPathInfo()
    {
        $request = $this->getEnv('PATH_INFO');
        if ($request === null) {
            $request = $this->getEnv('ORIG_PATH_INFO');
        }
        if ($request === null) {
            $request = '';
        }

        $query     = explode('/', trim($request, '/'));
        $path_info = array();
        for ($i = 0; $i < count($query); $i += 2) {
            $path_info[$query[$i]] = isset($query[$i + 1]) ? $query[$i + 1] : null;
        }

        return $path_info;
    }

    /**
     * Ķѿֵ
     * 
     * ʸƽ뤿ᡢͤƶǽ
     * 
     * @param  string $varkey
     * @return string|null
     * @access public
     */
    function getEnv($varkey)
    {
        $varvalue = null;
        if (isset($_SERVER[$varkey])) {
            $varvalue = Utility::to('EraseCtrlChar', $_SERVER[$varkey]);
        }

        return $varvalue;
    }

    /**
     * ꥯȤ줿ۥȥФֵ̾
     * - ꥯȥإåΡhostפ
     *  - HTTP/1.0Ǥ¸ߤʤ
     *  - HTTP/1.1ǤꥯȤˤäƤ¸ߤʤ
     *  - ꥯȤ˥ݡֹ椬Хݡֹޤ
     * 
     * @return string
     * @access public
     */
    function getHost()
    {
        $host = $this->getEnv('HTTP_X_FORWARDED_HOST');
        if ($host === null) {
            $host = $this->getEnv('HTTP_HOST');
        }
        if ($host === null) {
            $host = '';
        }

        return $host;
    }

    /**
     * ꥽ΥۥȡФֵ̾
     * - ФꤵƤ̾
     *  - ApacheǤСServerNameץǥ쥯ƥ֤
     *  - СۥȤǤϥФˤäƤ
     *    Request::getHost()ͤȰۤʤǽ
     * 
     * @return string
     * @access public
     */
    function getServerName()
    {
        $server_name = $this->getEnv('HTTP_X_FORWARDED_SERVER');
        if ($server_name === null) {
            $server_name = $this->getEnv('SERVER_NAME');
        }
        if ($server_name === null) {
            $server_name = '';
        }

        return $server_name;
    }

    /**
     * ե顼ֵ
     * 
     * @return string
     * @access public
     */
    function getReferer()
    {
        $referer = $this->getEnv('HTTP_REFERER');
        if ($referer === null) {
            $referer = '';
        }

        return $referer;
    }

    /**
     * ¹ԤƤ륹ץȤΥɥȥ롼ȤΥѥֵ
     * 
     * @return string
     * @access public
     */
    function getScriptName()
    {
        $script_name = $this->getEnv('SCRIPT_NAME');
        if ($script_name === null) {
            $script_name = $this->getEnv('ORIG_SCRIPT_NAME');
        }
        if ($script_name === null) {
            $script_name = '';
        }

        return $script_name;
    }

    /**
     * ¹ԤƤmainȤʤäƤ˥ץȤΥѥֵ
     * 
     * @return string
     * @access public
     */
    function getMainPath()
    {
        $script_filename = $this->getEnv('SCRIPT_FILENAME');
        if ($script_filename === null) {
            $script_filename = $this->getEnv('ORIG_SCRIPT_FILENAME');
        }
        if ($script_filename === null) {
            $script_filename = '';
        }

        return $script_filename;
    }

    /**
     * ꥽Υֵ
     * - http:
     * - https:
     * 
     * @return string
     * @access public
     */
    function getScheme()
    {
        $scheme = $this->isSecure() ? 'https:' : 'http:';

        return $scheme;
    }

    /**
     * ꥽ΥȥۥȤֵ̾
     * - http://example.com
     * - https://example.com
     * - http://example.com:8080
     * 
     * @return string
     * @access public
     * @todo ͥŪˡHTTP_HOSTפͤѤΤǡ
     *       ݡֹ椬ȤƤȡhttp://example.com:8080:8080
     *       ȤֵͤѤǽΤ׽
     */
    function getHostUri()
    {
        $host_name = $this->getHost();
        if ($host_name === '') {
            $host_name = $this->getServerName();
        }

        $host_uri = $this->getScheme().'//'.$host_name;
        $std_port = $this->isSecure() ? '443' : '80';
        $port_no  = $this->getEnv('SERVER_PORT');
        if (isset($port_no) && ($port_no !== '') && ($port_no !== $std_port)) {
            $host_uri .= ':'.$port_no;
        }

        return $host_uri;
    }

    /**
     * ꥽Υѥֵ
     * - /foo/bar/hoge.html
     * - /foo/bar/
     * 
     * @return string
     * @access public
     */
    function getPathUri()
    {
        $path = $this->getEnv('REQUEST_URI');
        if ($path === null) {
            $path = '';
        }

        if (preg_match('/^(?:http)(?:s)?:\/\//', $path)) {
            $url  = parse_url($path);
            $path = $url['path'];
        }
        if (($i = strpos($path, '?')) !== false) {
            $path = substr($path, 0, $i);
        }

        return $path;
    }

    /**
     * ꥽URLֵ
     * - http://example.com/
     * - https://example.com/foo/bar/
     * - http://example.com:8080/foo/bar/hoge.html
     * 
     * @return string
     * @access public
     */
    function getUrl()
    {
        return 
            $this->getHostUri().$this->getPathUri();
    }

    /**
     * ꥯȡ᥽åɤֵ
     * 
     * бƤʤ᥽åɤǤϡGETפֵ
     * бƤ᥽åɤϰʲΤȤ
     * 1. GET
     * 2. POST
     * 3. PUT
     * 4. DELETE
     * 5. HEAD
     *
     * @return string
     * @access public
     */
    function getMethod()
    {
        $method = $this->getEnv('REQUEST_METHOD');
        switch ($method) {
        case 'GET' : 
        case 'POST' : 
        case 'PUT' : 
        case 'DELETE' : 
        case 'HEAD' : 
            break;
        default : 
            $method = 'GET';
        }

        return $method;
    }

    /**
     * ꥯȤΥ᥽åɤGETפ򸡺
     *
     * @return boolean
     * @access public
     */
    function isGet()
    {
        return 
            ($this->getMethod() === 'GET');
    }

    /**
     * ꥯȤΥ᥽åɤPOSTפ򸡺
     *
     * @return boolean
     * @access public
     */
    function isPost()
    {
        return 
            ($this->getMethod() === 'POST');
    }

    /**
     * ꥯȤΥ᥽åɤPUTפ򸡺
     *
     * @return boolean
     * @access public
     */
    function isPut()
    {
        return 
            ($this->getMethod() === 'PUT');
    }

    /**
     * ꥯȤΥ᥽åɤDELETEפ򸡺
     *
     * @return boolean
     * @access public
     */
    function isDelete()
    {
        return 
            ($this->getMethod() === 'DELETE');
    }

    /**
     * ꥯȤΥ᥽åɤHEADפ򸡺
     *
     * @return boolean
     * @access public
     */
    function isHead()
    {
        return 
            ($this->getMethod() === 'HEAD');
    }

    /**
     * ꥯȤHTTPSץȥˤŹ̿򸡺
     *
     * @return boolean
     * @access public
     */
    function isSecure()
    {
        return 
            ($this->getEnv('HTTPS') === 'on') || 
            ($this->getEnv('HTTP_X_FORWARDED_PROTO') === 'https');
    }

    /**
     * GET᥽åɤΥꥯѿ
     * 
     * @access private
     * @todo   ᡼ܥΥåͤ򥹥ޡȤ˼
     *         if (strrpos($varkey, '_x') === (strlen($varkey) - 2))
     *         if (strrpos($varkey, '_y') === (strlen($varkey) - 2))
     */
    function _fetchGetParameters()
    {
        foreach ($_GET as $varkey => $varvalue) {
            $this->_parameters[$varkey] = $varvalue;
        }
    }

    /**
     * POST᥽åɤΥꥯѿ
     * 
     * @return void
     * @access private
     * @todo   ᡼ܥΥåͤ򥹥ޡȤ˼
     *         if (strrpos($varkey, '_x') === (strlen($varkey) - 2))
     *         if (strrpos($varkey, '_y') === (strlen($varkey) - 2))
     */
    function _fetchPostParameters()
    {
        foreach ($_POST as $varkey => $varvalue) {
            $this->_parameters[$varkey] = $varvalue;
        }
    }
}
